@@ -1,11 +1,11 @@
 use crate::util::extract_doc_lines;
 use darling::FromMeta;
-use proc_macro2::TokenStream;
+use proc_macro2::{Ident, TokenStream};
 use quote::{quote, ToTokens};
 use std::collections::HashMap;
-use syn::{AttributeArgs, Expr, ItemEnum, ItemStruct, ItemType, Lit, Type, GenericParam};
+use syn::{AttributeArgs, Expr, GenericParam, ItemEnum, ItemStruct, ItemType, Lit, Type};
 
-#[derive(Debug, FromMeta)]
+#[derive(Debug, FromMeta, Clone)]
 pub struct FFITypeAttributes {
     #[darling(default)]
     opaque: bool,
@@ -13,15 +13,44 @@ pub struct FFITypeAttributes {
     #[darling(default)]
     surrogates: HashMap<String, String>,
 
+    #[darling(default)]
+    patterns: HashMap<String, ()>,
+
     #[darling(default)]
     skip: HashMap<String, ()>,
+
+    #[darling(default)]
+    name: Option<String>,
+}
+
+fn derive_variant_info(item: ItemEnum, idents: &[Ident], names: &[String], values: &[i32], docs: &[String]) -> TokenStream {
+    let span = item.ident.span();
+    let name = item.ident.to_string();
+    let name_ident = syn::Ident::new(&name, span);
+
+    quote! {
+        impl interoptopus::lang::rust::VariantInfo for #name_ident {
+            fn variant_info(&self) -> interoptopus::lang::c::Variant {
+                match self {
+                    #(
+                       Self::#idents => {
+                            let documentation = interoptopus::lang::c::Documentation::from_line(#docs);
+                            interoptopus::lang::c::Variant::new(#names.to_string(), #values as usize, documentation)
+                       },
+                    )*
+                }
+            }
+        }
+    }
 }
 
 pub fn ffi_type_enum(attr: FFITypeAttributes, input: TokenStream, item: ItemEnum) -> TokenStream {
+    let span = item.ident.span();
     let name = item.ident.to_string();
-    let name_ident = syn::Ident::new(&name, item.ident.span());
+    let name_ident = syn::Ident::new(&name, span);
 
     let mut variant_names = Vec::new();
+    let mut variant_idents = Vec::new();
     let mut variant_values = Vec::new();
     let mut variant_docs = Vec::new();
 
@@ -29,25 +58,21 @@ pub fn ffi_type_enum(attr: FFITypeAttributes, input: TokenStream, item: ItemEnum
 
     let doc_line = extract_doc_lines(&item.attrs).join("\n");
 
-    for variant in item.variants {
+    for variant in &item.variants {
         let ident = variant.ident.to_string();
         let variant_doc_line = extract_doc_lines(&variant.attrs).join("\n");
 
-        let this_id = if let Some((_, e)) = variant.discriminant {
+        let this_id = if let Some((_, e)) = &variant.discriminant {
             match e {
-                Expr::Lit(e) => match e.lit {
+                Expr::Lit(e) => match &e.lit {
                     Lit::Int(x) => {
                         let number = x.base10_parse().expect("Must be number");
                         next_id = number + 1;
                         number
                     }
-                    _ => {
-                        panic!("Unknown token.")
-                    }
+                    _ => panic!("Unknown token."),
                 },
-                _ => {
-                    panic!("Unknown token.")
-                }
+                _ => panic!("Unknown token."),
             }
         } else {
             let id = next_id;
@@ -56,29 +81,42 @@ pub fn ffi_type_enum(attr: FFITypeAttributes, input: TokenStream, item: ItemEnum
         };
 
         if !attr.skip.contains_key(&ident) {
+            variant_idents.push(syn::Ident::new(&ident, span));
             variant_names.push(ident);
             variant_values.push(this_id);
             variant_docs.push(variant_doc_line);
         }
     }
 
+    let variant_infos = derive_variant_info(item.clone(), &variant_idents, &variant_names, &variant_values, &variant_docs);
+    let ctype_info_return = if attr.patterns.contains_key("success_enum") {
+        quote! {
+            let success_variant = Self::SUCCESS.variant_info();
+            let the_success_enum = interoptopus::patterns::success_enum::SuccessEnum::new(rval, success_variant);
+            let the_pattern = interoptopus::patterns::TypePattern::SuccessEnum(the_success_enum);
+            interoptopus::lang::c::CType::Pattern(the_pattern)
+        }
+    } else {
+        quote! { interoptopus::lang::c::CType::Enum(rval) }
+    };
+
     quote! {
         #input
 
+        #variant_infos
+
         impl interoptopus::lang::rust::CTypeInfo for #name_ident {
             fn type_info() -> interoptopus::lang::c::CType {
+                use interoptopus::lang::rust::VariantInfo;
 
                 let documentation = interoptopus::lang::c::Documentation::from_line(#doc_line);
-
                 let mut rval = interoptopus::lang::c::EnumType::new(#name.to_string(), documentation);
 
                 #({
-                    let documentation = interoptopus::lang::c::Documentation::from_line(#variant_docs);
-                    rval.add_variant(interoptopus::lang::c::Variant::new(#variant_names.to_string(), #variant_values as usize, documentation));
+                    rval.add_variant(Self::#variant_idents.variant_info());
                 })*
 
-
-                interoptopus::lang::c::CType::Enum(rval)
+                #ctype_info_return
             }
         }
     }
@@ -87,6 +125,7 @@ pub fn ffi_type_enum(attr: FFITypeAttributes, input: TokenStream, item: ItemEnum
 pub fn ffi_type_struct(attr: FFITypeAttributes, input: TokenStream, item: ItemStruct) -> TokenStream {
     let name = item.ident.to_string();
     let name_ident = syn::Ident::new(&name, item.ident.span());
+    let c_struct_name = attr.name.unwrap_or(name);
 
     let mut generic_params = Vec::new();
     let mut field_names = Vec::new();
@@ -94,8 +133,8 @@ pub fn ffi_type_struct(attr: FFITypeAttributes, input: TokenStream, item: ItemSt
     let mut field_docs = Vec::new();
     let mut generic_params_needing_bounds = Vec::new();
 
-    let mut generics_params = quote! { };
-    let mut where_clause = quote! { };
+    let mut generics_params = quote! {};
+    let mut where_clause = quote! {};
 
     let doc_line = extract_doc_lines(&item.attrs).join("\n");
 
@@ -115,7 +154,7 @@ pub fn ffi_type_struct(attr: FFITypeAttributes, input: TokenStream, item: ItemSt
         generics_params = quote! { < #(#generic_params),* > };
 
         if !generic_params_needing_bounds.is_empty() {
-            if let Some(x) =  item.generics.where_clause {
+            if let Some(x) = item.generics.where_clause {
                 // where_clause = quote! { #x, #( #generic_params_needing_bounds: interoptopus::lang::rust::CTypeInfo)*, }
                 where_clause = quote! { #x }
             } else {
@@ -161,7 +200,7 @@ pub fn ffi_type_struct(attr: FFITypeAttributes, input: TokenStream, item: ItemSt
 
                 let documentation = interoptopus::lang::c::Documentation::from_line(#doc_line);
 
-                let mut rval = interoptopus::lang::c::CompositeType::with_documentation(#name.to_string(), documentation);
+                let mut rval = interoptopus::lang::c::CompositeType::with_documentation(#c_struct_name.to_string(), documentation);
 
                 #({
                     let documentation = interoptopus::lang::c::Documentation::from_line(#field_docs);
@@ -194,7 +233,6 @@ pub fn ffi_type_struct_opqaue(_attr: FFITypeAttributes, input: TokenStream, item
     }
 }
 
-
 pub fn ffi_type(attr: AttributeArgs, input: TokenStream) -> TokenStream {
     let ffi_attributes: FFITypeAttributes = FFITypeAttributes::from_list(&attr).unwrap();
 
